@@ -25,7 +25,7 @@
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4panel/xfce-panel-plugin.h>
 
 #include <stdlib.h>
@@ -39,6 +39,11 @@
 #include <errno.h>
 #include <sys/stat.h>
 
+#ifdef LIBXFCE4PANEL_CHECK_VERSION
+#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
+#define HAS_PANEL_49
+#endif
+#endif
 
 #define PLUGIN_NAME	"DiskPerf"
 #define BORDER          8
@@ -48,6 +53,8 @@
     data, but only a single value combining both */
 #if  defined(__NetBSD__)
 #define	SEPARATE_BUSY_TIMES	0
+#elif  defined(__sun__)
+#define	SEPARATE_BUSY_TIMES	0
 #elif defined(__linux__)
 #define	SEPARATE_BUSY_TIMES	1
 #else
@@ -79,7 +86,7 @@ typedef enum monitor_bar_order_t {
 typedef struct param_t {
     /* Configurable parameters */
     char            acDevice[64];
-#if  !defined(__NetBSD__) && !defined(__OpenBSD__)
+#if  !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(__sun__)
     dev_t           st_rdev;
 #endif
     int             fTitleDisplayed;
@@ -159,7 +166,7 @@ static int DisplayPerf (struct diskperf_t *p_poPlugin)
     rbytes = wbytes = iRBusy_ns = iWBusy_ns = -1;
     memset (&oPerf, 0, sizeof (oPerf));
     oPerf.qlen = -1;
-#if defined (__NetBSD__) || defined(__OpenBSD__)
+#if defined (__NetBSD__) || defined(__OpenBSD__) || defined(__sun__)
     status = DevGetPerfData (poConf->acDevice, &oPerf);
 #else
     status = DevGetPerfData (&(poConf->st_rdev), &oPerf);
@@ -206,18 +213,22 @@ static int DisplayPerf (struct diskperf_t *p_poPlugin)
 	     "  Write :%3.2f\n"
 	     "  Total :%3.2f\n"
 	     "Busy time (%c)\n"
+#if SEPARATE_BUSY_TIMES
 	     "  Read : %3d\n"
 	     "  Write : %3d\n"
+#endif
 	     "  Total : %3d",
 	     poConf->acTitle,
 	     arPerf[R_DATA],
 	     arPerf[W_DATA],
 	     arPerf[RW_DATA],
 	     '%',
-	     SEPARATE_BUSY_TIMES && (oPerf.qlen >= 0) ?
+#if SEPARATE_BUSY_TIMES
+	     (oPerf.qlen >= 0) ?
 	     (int) round(arBusy[R_DATA]) : -1,
-	     SEPARATE_BUSY_TIMES && (oPerf.qlen >= 0) ?
+	     (oPerf.qlen >= 0) ?
 	     (int) round(arBusy[W_DATA]) : -1,
+#endif
 	     (oPerf.qlen >= 0) ? (int) round(arBusy[RW_DATA]) : -1);
     gtk_tooltips_set_tip (s_poToolTips, GTK_WIDGET (poMonitor->wEventBox),
 			  acToolTips, 0);
@@ -289,18 +300,18 @@ static int SetSingleBarColor (struct diskperf_t *p_poPlugin, int p_iBar)
     struct diskperf_t *poPlugin = p_poPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
-    GtkRcStyle     *poStyle;
     Widget_t       *pwBar;
 
     pwBar = poMonitor->aoPerfBar[p_iBar].pwBar;
-    poStyle = gtk_widget_get_modifier_style (GTK_WIDGET (*pwBar));
-    if (!poStyle)
-	poStyle = gtk_rc_style_new ();
-    else {
-	poStyle->color_flags[GTK_STATE_PRELIGHT] |= GTK_RC_BG;
-	poStyle->bg[GTK_STATE_PRELIGHT] = poConf->aoColor[p_iBar];
-    }
-    gtk_widget_modify_style (GTK_WIDGET (*pwBar), poStyle);
+	gtk_widget_modify_bg(GTK_WIDGET(*pwBar),
+						 GTK_STATE_PRELIGHT,
+						 &poConf->aoColor[p_iBar]);
+	gtk_widget_modify_bg(GTK_WIDGET(*pwBar),
+						 GTK_STATE_SELECTED,
+						 &poConf->aoColor[p_iBar]);
+	gtk_widget_modify_base(GTK_WIDGET(*pwBar),
+						 GTK_STATE_SELECTED,
+						 &poConf->aoColor[p_iBar]);
     return (0);
 }				/* SetSingleBarColor() */
 
@@ -400,7 +411,7 @@ static diskperf_t *diskperf_create_control (XfcePanelPlugin *plugin)
     struct diskperf_t *poPlugin;
     struct param_t *poConf;
     struct monitor_t *poMonitor;
-#if !defined(__NetBSD__) && !defined(__OpenBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(__sun__)
     struct stat     oStat;
     int             status;
 #endif
@@ -415,6 +426,9 @@ static diskperf_t *diskperf_create_control (XfcePanelPlugin *plugin)
 #if defined(__NetBSD__) || defined(__OpenBSD__)
     strncpy (poConf->acDevice, "wd0", 64);
     strncpy (poConf->acTitle, "wd0", 16);
+#elif defined(__sun__)
+    strncpy (poConf->acDevice, "sd0", 64);
+    strncpy (poConf->acTitle, "sd0", 16);
 #else
     strncpy (poConf->acDevice, "/dev/sda", 64);
     status = stat (poConf->acDevice, &oStat);
@@ -483,7 +497,7 @@ static void diskperf_read_config (XfcePanelPlugin *plugin,
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     Widget_t       *pw2ndBar = poPlugin->oMonitor.awProgressBar + 1;
-#if !defined(__NetBSD__) && !defined(__OpenBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(__sun__)
     struct stat     oStat;
     int             status;
 #endif
@@ -500,7 +514,7 @@ static void diskperf_read_config (XfcePanelPlugin *plugin,
     if ((value = xfce_rc_read_entry (rc, (CONF_DEVICE), NULL))) {
         memset (poConf->acDevice, 0, sizeof (poConf->acDevice));
         strncpy (poConf->acDevice, value, sizeof (poConf->acDevice) - 1);
-#if !defined(__NetBSD__) && !defined(__OpenBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(__sun__)
         status = stat (poConf->acDevice, &oStat);
         poConf->st_rdev = (status == -1 ? 0 : oStat.st_rdev);
 #endif
@@ -624,13 +638,14 @@ static void SetDevice (Widget_t p_wTF, void *p_pvPlugin)
     struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     const char     *pcDevice = gtk_entry_get_text (GTK_ENTRY (p_wTF));
-#if !defined(__NetBSD__) && !defined(__OpenBSD__)
+#if !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(__sun__)
     struct stat     oStat;
     int             status;
 
     status = stat (pcDevice, &oStat);
     if (status == -1) {
-	xfce_err ("%s\n"
+	xfce_dialog_show_error (NULL, NULL,
+                  "%s\n"
 		  "%s: %s (%d)",
 		  PLUGIN_NAME, pcDevice, strerror (errno), errno);
 	return;
@@ -648,7 +663,6 @@ static void ToggleTitle (Widget_t p_w, void *p_pvPlugin)
 {
     struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
-    struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
 
     poConf->fTitleDisplayed =
@@ -852,7 +866,8 @@ static int CheckStatsAvailability ()
 	return (0);
     if (status < 0) {
 	status *= -1;
-	xfce_err (_("%s\n"
+	xfce_dialog_show_error (NULL, NULL, 
+                  _("%s\n"
 		  "%s: %s (%d)\n\n"
 		  "This monitor will not work!\n"
 		  "Please remove it."),
@@ -862,14 +877,16 @@ static int CheckStatsAvailability ()
     }
     switch (status) {
 	case NO_EXTENDED_STATS:
-	    xfce_err (_("%s: No disk extended statistics found!\n"
+	    xfce_dialog_show_error (NULL, NULL, 
+                      _("%s: No disk extended statistics found!\n"
 		      "Either old kernel (< 2.4.20) or not\n"
 		      "compiled with CONFIG_BLK_STATS turned on.\n\n"
 		      "This monitor will not work!\n"
 		      "Please remove it."), PLUGIN_NAME);
 	    break;
 	default:
-	    xfce_err (_("%s: Unknown error\n\n"
+	    xfce_dialog_show_error (NULL, NULL, 
+                      _("%s: Unknown error\n\n"
 		      "This monitor will not work!\n"
 		      "Please remove it."), PLUGIN_NAME);
     }
@@ -881,13 +898,17 @@ static int CheckStatsAvailability ()
 static void About (Widget_t w, void *unused)
 	/* Called back when the About button in clicked */
 {
-    xfce_info (_("%s %s - Disk Performance Monitor\n"
-	       "Display instantaneous disk I/O transfer rates and busy times "
-	       "on Linux and NetBSD systems\n\n"
-	       "(c) 2003, 2004 Roger Seguin <roger_seguin@msn.com>\n"
-	       "NetBSD statistics collection: (c) 2003 Benedikt Meurer\n"
-	       "\t<benedikt.meurer@unix-ag.uni-siegen.de>"),
-	       PACKAGE, VERSION);
+   const gchar *auth[] = { "Roger Seguin <roger_seguin@msn.com>",
+	"NetBSD statistics collection: (c) 2003 Benedikt Meurer <benedikt.meurer@unix-ag.uni-siegen.de>",
+	"Solaris statistics collection: (c) 2011 Peter Tribble <peter.tribble@gmail.com>", NULL };
+   gtk_show_about_dialog(NULL,
+      "license", xfce_get_license_text (XFCE_LICENSE_TEXT_BSD),
+      "version", PACKAGE_VERSION,
+      "program-name", PACKAGE_NAME,
+      "comments", _("Diskperf monitor displays instantaneous disk I/O transfer rates and busy times"),
+      "website", "http://goodies.xfce.org/projects/panel-plugins/xfce4-diskperf-plugin",
+      "copyright", _("Copyright (c) 2003, 2004 Roger Seguin"),
+      "authors", auth, NULL);
 }				/* About() */
 
 	/**************************************************************/
@@ -1055,9 +1076,42 @@ static gboolean diskperf_set_size (XfcePanelPlugin *plugin, int p_size,
     return TRUE;
 }				/* diskperf_set_size() */
 
+#ifdef HAS_PANEL_49
 	/**************************************************************/
 
-static void diskperf_set_orientation (XfcePanelPlugin *plugin, 
+static void diskperf_set_mode (XfcePanelPlugin *plugin,
+                               XfcePanelPluginMode p_iMode,
+                               diskperf_t *poPlugin)
+	/* Plugin API */
+	/* Invoked when the panel changes mode */
+{
+    struct monitor_t *poMonitor = &(poPlugin->oMonitor);
+    GtkOrientation    p_iOrientation;
+
+    TRACE ("diskperf_set_mode()\n");
+
+    p_iOrientation =
+      (p_iMode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL) ?
+      GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
+
+
+    if (poPlugin->iTimerId) {
+	g_source_remove (poPlugin->iTimerId);
+	poPlugin->iTimerId = 0;
+    }
+    gtk_container_remove (GTK_CONTAINER (poMonitor->wEventBox),
+			  GTK_WIDGET (poMonitor->wBox));
+    CreateMonitorBars (poPlugin, p_iOrientation);
+    SetTimer (poPlugin);
+    gtk_label_set_angle (GTK_LABEL (poMonitor->wTitle),
+                         (p_iMode != XFCE_PANEL_PLUGIN_MODE_VERTICAL) ?
+                         0 : 270);
+    diskperf_set_size (plugin, xfce_panel_plugin_get_size (plugin), poPlugin);
+}				/* diskperf_set_orientation() */
+#else
+	/**************************************************************/
+
+static void diskperf_set_orientation (XfcePanelPlugin *plugin,
                                       GtkOrientation p_iOrientation,
                                       diskperf_t *poPlugin)
 	/* Plugin API */
@@ -1074,13 +1128,15 @@ static void diskperf_set_orientation (XfcePanelPlugin *plugin,
 			  GTK_WIDGET (poMonitor->wBox));
     CreateMonitorBars (poPlugin, p_iOrientation);
     SetTimer (poPlugin);
+    diskperf_set_size (plugin, xfce_panel_plugin_get_size (plugin), poPlugin);
 }				/* diskperf_set_orientation() */
-
+#endif
 	/**************************************************************/
 
 static void diskperf_construct (XfcePanelPlugin *plugin)
 {
     diskperf_t *diskperf = diskperf_create_control (plugin);
+    xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
     g_signal_connect (plugin, "free-data", G_CALLBACK (diskperf_free), 
                       diskperf);
@@ -1091,8 +1147,13 @@ static void diskperf_construct (XfcePanelPlugin *plugin)
     g_signal_connect (plugin, "size-changed", G_CALLBACK (diskperf_set_size), 
                       diskperf);
 
-    g_signal_connect (plugin, "orientation-changed", 
+#ifdef HAS_PANEL_49
+    g_signal_connect (plugin, "mode-changed",
+                      G_CALLBACK (diskperf_set_mode), diskperf);
+#else
+    g_signal_connect (plugin, "orientation-changed",
                       G_CALLBACK (diskperf_set_orientation), diskperf);
+#endif
 
     xfce_panel_plugin_menu_show_about (plugin);
     g_signal_connect (plugin, "about", G_CALLBACK (About), NULL);
